
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <sys/un.h>

#include "hin.h"
#include "http/http.h"
#include "http/cache.h"
#include "conf.h"

#include "fcgi.h"

static hin_fcgi_group_t * fcgi_group = NULL;

static void hin_fcgi_group_clean (hin_fcgi_group_t * fcgi) {
  basic_dlist_t * elem = fcgi->idle_worker.next;
  while (elem) {
    basic_dlist_t * next = elem->next;
    hin_fcgi_worker_t * worker = basic_dlist_ptr (elem, offsetof (hin_fcgi_worker_t, list));
    hin_cgi_worker_free (worker);
    elem = next;
  }

  for (int i=0; i < fcgi->max_socket; i++) {
    hin_fcgi_socket_t * socket = fcgi->socket[i];
    if (socket) {
      socket->fcgi = NULL;
    }
  }

  if (fcgi->socket) free (fcgi->socket);
  if (fcgi->host) free (fcgi->host);
  if (fcgi->port) free (fcgi->port);
  if (fcgi->uri) free (fcgi->uri);
}

void hin_fcgi_clean () {
  hin_fcgi_group_t * fcgi = fcgi_group;
  while (fcgi) {
    hin_fcgi_group_t * next = fcgi->next;
    hin_fcgi_group_clean (fcgi);
    free (fcgi);
    fcgi = next;
  }
  fcgi_group = NULL;
}

hin_fcgi_group_t * hin_fcgi_start (const char * uri, int min, int max) {
  string_t source, host, port;
  source.ptr = (char*)uri;
  source.len = strlen (source.ptr);
  if (match_string (&source, "tcp://") <= 0) {
    hin_error ("fcgi missing 'tcp://' '%s'", uri);
    return NULL;
  }

  if (match_string (&source, "([%w]+)", &host) <= 0) {
    hin_error ("fcgi missing host '%s'", uri);
    return NULL;
  }
  if (match_string (&source, ":([%d]+)", &port) <= 0) {
    hin_error ("fcgi missing port '%s'", uri);
    return NULL;
  }

  hin_fcgi_group_t * fcgi = calloc (1, sizeof (*fcgi));
  fcgi->host = strndup (host.ptr, host.len);
  fcgi->port = strndup (port.ptr, port.len);
  fcgi->uri = strdup (uri);
  fcgi->magic = HIN_FCGI_MAGIC;
  fcgi->min_socket = min;
  fcgi->max_socket = max;
  if (max) {
    fcgi->socket = calloc (1, sizeof (void*) * max);
  }

  if (hin_g.debug & (HNDBG_CGI|HNDBG_CONFIG))
    hin_debug ("fcgi group '%s' min %d max %d\n", uri, min, max);

  fcgi->next = fcgi_group;
  fcgi_group = fcgi;

  return fcgi;
}

int hin_fastcgi (httpd_client_t * http, void * fcgi1, const char * script_path, const char * path_info) {
  if (http->state & HIN_REQ_DATA) return -1;
  http->state |= (HIN_REQ_DATA | HIN_REQ_FCGI);

  hin_fcgi_group_t * fcgi = fcgi1;
  if (fcgi == NULL || fcgi->magic != HIN_FCGI_MAGIC) {
    httpd_error (http, 500, "fcgi invalid");
    return -1;
  }

  if (!HIN_HTTPD_CGI_CHUNKED_UPLOAD && (http->peer_flags & HIN_HTTP_CHUNKED_UPLOAD)) {
    httpd_error (http, 411, "cgi spec denies chunked upload");
    return -1;
  }

  if (hin_cache_check (NULL, http) > 0) {
    return 0;
  }

  hin_fcgi_worker_t * worker = hin_fcgi_get_worker (fcgi);
  if (worker == NULL) {
    httpd_error (http, 500, "worker null");
    return -1;
  }
  worker->http = http;
  httpd_request_chunked (http);

  hin_fcgi_worker_request (worker);

  return 0;
}

