#ifndef HIN_FCGI_H
#define HIN_FCGI_H

#include "hin.h"

#include "basic_lists.h"

#define FCGI_VERSION_1           1

#define FCGI_BEGIN_REQUEST       1
#define FCGI_ABORT_REQUEST       2
#define FCGI_END_REQUEST         3
#define FCGI_PARAMS              4
#define FCGI_STDIN               5
#define FCGI_STDOUT              6
#define FCGI_STDERR              7
#define FCGI_DATA                8
#define FCGI_GET_VALUES          9
#define FCGI_GET_VALUES_RESULT  10
#define FCGI_UNKNOWN_TYPE       11
#define FCGI_MAXTYPE (FCGI_UNKNOWN_TYPE)

#define FCGI_RESPONDER  1
#define FCGI_AUTHORIZER 2
#define FCGI_FILTER     3

#define FCGI_REQUEST_COMPLETE 0
#define FCGI_CANT_MPX_CONN    1
#define FCGI_OVERLOADED       2
#define FCGI_UNKNOWN_ROLE     3

#define FCGI_PADDING (sizeof (uintptr_t))
#define FCGI_ROUND_TO_PAD(num) ((((num-1) / FCGI_PADDING) + 1) * FCGI_PADDING)

enum { HIN_FCGI_DO_INIT = 0x1, HIN_FCGI_BUSY = 0x2 };

typedef struct __attribute__((__packed__)) {
  uint8_t version;
  uint8_t type;
  uint16_t request_id;
  uint16_t length;
  uint8_t padding;
  uint8_t reserved;
  uint8_t data[];
} FCGI_Header;

typedef struct __attribute__((__packed__)) {
  uint8_t version;
  uint8_t type;
  uint16_t request_id;
  uint16_t length;
  uint8_t padding;
  uint8_t reserved;
} FCGI_Header_int;

typedef struct __attribute__((__packed__)) {
  uint16_t role;
  uint8_t flags;
  uint8_t reserved[5];
} FCGI_BeginRequestBody;

typedef struct __attribute__((__packed__)) {
  FCGI_Header_int header;
  FCGI_BeginRequestBody body;
} FCGI_BeginRequestRecord;

typedef struct __attribute__((__packed__)) {
  uint32_t appStatus;
  uint8_t protocolStatus;
  uint8_t reserved[3];
} FCGI_EndRequestBody;

enum { HIN_FCGI_SOCKET_REUSE = 0x1, };

typedef struct {
  int req_id;
  uint32_t io_state;
  httpd_client_t * http;

  hin_buffer_t * header_buf;
  hin_pipe_t * out;

  basic_dlist_t list;

  struct hin_fcgi_socket_struct * socket;
} hin_fcgi_worker_t;

typedef struct hin_fcgi_socket_struct {
  int fd, id;
  uint32_t cflags;
  uint32_t flags;

  struct sockaddr ai_addr;
  socklen_t ai_addrlen;

  hin_fcgi_worker_t ** worker;
  int last_worker, num_worker, max_worker;

  basic_dlist_t que;
  hin_buffer_t * read_buffer;

  struct hin_fcgi_group_struct * fcgi;
} hin_fcgi_socket_t;

typedef struct hin_fcgi_group_struct {
  char * host;
  char * port;
  char * uri;

  int cur_socket, min_socket, max_socket;
  hin_fcgi_socket_t ** socket;

  uint32_t magic;

  basic_dlist_t idle_worker;

  struct hin_fcgi_group_struct * next;
} hin_fcgi_group_t;

// fcgi internal
FCGI_Header * hin_fcgi_header (hin_buffer_t * buf, int type, int id, int sz);
int hin_fcgi_write_request (hin_fcgi_worker_t * worker);

hin_fcgi_socket_t * hin_fcgi_get_socket (hin_fcgi_group_t * fcgi);
void hin_fcgi_socket_close (hin_fcgi_socket_t * socket);

hin_fcgi_worker_t * hin_fcgi_get_worker (hin_fcgi_group_t * fcgi_group);
int hin_fcgi_worker_reset (hin_fcgi_worker_t * worker);
int hin_fcgi_worker_request (hin_fcgi_worker_t * worker);
void hin_cgi_worker_free (hin_fcgi_worker_t * worker);

int hin_fcgi_socket_continue (hin_fcgi_socket_t * socket);

int hin_fcgi_write_post (hin_buffer_t * buf, hin_fcgi_worker_t * worker);

// cgi common
int hin_cgi_fast_worker_init_pipe (hin_fcgi_worker_t * worker);
int hin_cgi_old_worker_init_pipe (hin_fcgi_worker_t * worker, int fd);

int hin_cgi_location (httpd_client_t * http, hin_buffer_t * buf, string_t * origin);

#endif

